@@ -185,6 +185,7 @@ class Crate(SourceFetcher):
                 default_name = os.path.basename(url)
                 request = urllib.request.Request(url)
                 request.add_header('Accept', '*/*')
+                request.add_header('User-Agent', 'BuildStream/1')
 
                 # We do not use etag in case what we have in cache is
                 # not matching ref in order to be able to recover from
@@ -320,7 +321,6 @@ class CargoSource(Source):
         # The url before any aliasing
         #
         self.url = self.node_get_member(node, str, 'url', 'https://static.crates.io/crates')
-        self.ref = self.node_get_member(node, list, 'ref', None)
         self.cargo_lock = self.node_get_member(node, str, 'cargo-lock', 'Cargo.lock')
         self.vendor_dir = self.node_get_member(node, str, 'vendor-dir', 'crates')
 
@@ -329,7 +329,7 @@ class CargoSource(Source):
         # Needs to be marked here so that `track` can translate it later.
         self.mark_download_url(self.url)
 
-        self.crates = self._parse_crates(self.ref)
+        self.load_ref(node)
 
     def preflight(self):
         return
@@ -338,7 +338,7 @@ class CargoSource(Source):
         return [self.url, self.cargo_lock, self.vendor_dir, self.ref]
 
     def get_consistency(self):
-        if self.ref is None:
+        if not self.crates:
             return Consistency.INCONSISTENT
 
         consistency = Consistency.CACHED
@@ -347,15 +347,28 @@ class CargoSource(Source):
         return consistency
 
     def load_ref(self, node):
-        self.ref = self.node_get_member(node, list, 'ref', None)
-        self.crates = self._parse_crates(self.ref)
+        ref = self.node_get_member(node, list, 'ref', None)
+        self._recompute_crates(ref)
 
     def get_ref(self):
         return self.ref
 
     def set_ref(self, ref, node):
-        node['ref'] = self.ref = ref
-        self.crates = self._parse_crates(self.ref)
+        node['ref'] = ref
+        self._recompute_crates(ref)
+
+    def _recompute_crates(self, ref):
+        self.crates = self._parse_crates(ref)
+        if not self.crates:
+            self.ref = None
+        else:
+            self.ref = [
+                {
+                    "name": crate.name,
+                    "version": crate.version,
+                    "sha": crate.sha
+                } for crate in self.crates
+            ]
 
     def track(self, previous_sources_dir):
         new_ref = []
@@ -381,7 +394,8 @@ class CargoSource(Source):
                 continue
             new_ref += [{
                 'name': package['name'],
-                'version': str(package['version'])
+                'version': str(package['version']),
+                'sha': package.get('checksum')
             }]
 
         # Make sure the order we set it at track time is deterministic
@@ -389,6 +403,9 @@ class CargoSource(Source):
 
         # Download the crates and get their shas
         for crate_obj in new_ref:
+            if crate_obj['sha'] is not None:
+                continue
+
             crate = Crate(self, crate_obj['name'], crate_obj['version'])
 
             crate_url = crate._get_url()
